@@ -32,7 +32,7 @@
  * Always add new items before the last NULL.
  */
 
-compress_model_t compress_modules_cmds[] = {
+static compress_model_t compress_modules_cmds[] = {
 	{ "none" , 0, 0, 0, NULL },
 	{ "zlib" , 1, WITH_COMPRESS_ZLIB , 0, NULL },
 	{ "lz4"  , 2, WITH_COMPRESS_LZ4  , 0, NULL },
@@ -87,7 +87,10 @@ static int val_level(
 	int compress_model,
 	int compress_level)
 {
-	return compress_modules_cmds[compress_model].ops->val_level(knet_h, compress_level);
+	if (compress_modules_cmds[compress_model].ops->val_level != NULL) {
+		return compress_modules_cmds[compress_model].ops->val_level(knet_h, compress_level);
+	}
+	return 0;
 }
 
 /*
@@ -184,6 +187,47 @@ static int compress_load_lib(knet_handle_t knet_h, int cmp_model, int rate_limit
 	return 0;
 }
 
+static int compress_lib_test(knet_handle_t knet_h)
+{
+	int savederrno = 0;
+	unsigned char src[KNET_DATABUFSIZE];
+	unsigned char dst[KNET_DATABUFSIZE_COMPRESS];
+	ssize_t dst_comp_len = KNET_DATABUFSIZE_COMPRESS, dst_decomp_len = KNET_DATABUFSIZE;
+	unsigned int i;
+
+	memset(src, 0, KNET_DATABUFSIZE);
+	memset(dst, 0, KNET_DATABUFSIZE_COMPRESS);
+
+	/*
+	 * NOTE: we cannot use compress and decompress API calls due to locking
+	 * so we need to call directly into the modules
+	 */
+
+	if (compress_modules_cmds[knet_h->compress_model].ops->compress(knet_h, src, KNET_DATABUFSIZE, dst, &dst_comp_len) < 0) {
+		savederrno = errno;
+		log_err(knet_h, KNET_SUB_COMPRESS, "Unable to compress test buffer. Please check your compression settings: %s", strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	if (compress_modules_cmds[knet_h->compress_model].ops->decompress(knet_h, dst, dst_comp_len, src, &dst_decomp_len) < 0) {
+		savederrno = errno;
+		log_err(knet_h, KNET_SUB_COMPRESS, "Unable to decompress test buffer. Please check your compression settings: %s", strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	for (i = 0; i < KNET_DATABUFSIZE; i++) {
+		if (src[i] != 0) {
+			log_err(knet_h, KNET_SUB_COMPRESS, "Decompressed buffer contains incorrect data");
+			errno = EINVAL;
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
 int compress_init(
 	knet_handle_t knet_h)
 {
@@ -277,15 +321,22 @@ int compress_cfg(
 			goto out_unlock;
 		}
 
+		knet_h->compress_model = cmp_model;
+		knet_h->compress_level = knet_handle_compress_cfg->compress_level;
+
+		if (compress_lib_test(knet_h) < 0) {
+			savederrno = errno;
+			err = -1;
+			goto out_unlock;
+		}
+
 out_unlock:
 		pthread_rwlock_unlock(&shlib_rwlock);
 	}
 
-	if (!err) {
-		knet_h->compress_model = cmp_model;
-		knet_h->compress_level = knet_handle_compress_cfg->compress_level;
-	} else {
+	if (err) {
 		knet_h->compress_model = 0;
+		knet_h->compress_level = 0;
 	}
 
 	errno = savederrno;
@@ -426,5 +477,7 @@ int knet_get_compress_list(struct knet_compress_info *compress_list, size_t *com
 	}
 	*compress_list_entries = outidx;
 
+	if (!err)
+		errno = 0;
 	return err;
 }
