@@ -24,7 +24,7 @@
  * internal module switch data
  */
 
-crypto_model_t crypto_modules_cmds[] = {
+static crypto_model_t crypto_modules_cmds[] = {
 	{ "nss", WITH_CRYPTO_NSS, 0, NULL },
 	{ "openssl", WITH_CRYPTO_OPENSSL, 0, NULL },
 	{ NULL, 0, 0, NULL }
@@ -208,5 +208,7 @@ int knet_get_crypto_list(struct knet_crypto_info *crypto_list, size_t *crypto_li
 	}
 	*crypto_list_entries = outidx;
 
+	if (!err)
+		errno = 0;
 	return err;
 }
