@@ -14,6 +14,7 @@
 #include <time.h>
 #include <netinet/in.h>
 #include <unistd.h>
+#include <limits.h>
 
 /**
  * @file libknet.h
@@ -23,6 +24,8 @@
  * Kronosnet is an advanced VPN system for High Availability applications.
  */
 
+#define KNET_API_VER 1
+
 /*
  * libknet limits
  */
@@ -98,7 +101,7 @@ typedef struct knet_handle *knet_handle_t;
  *
  * host_id  - Each host in a knet is identified with a unique
  *            ID. when creating a new handle local host_id
- *            must be specified (0 to UINT16T_MAX are all valid).
+ *            must be specified (0 to UINT16_MAX are all valid).
  *            It is the user's responsibility to check that the value
  *            is unique, or bad things might happen.
  *
@@ -109,6 +112,9 @@ typedef struct knet_handle *knet_handle_t;
  *            Make sure to either read from this filedescriptor properly and/or
  *            mark it O_NONBLOCK, otherwise if the fd becomes full, libknet could
  *            block.
+ *            It is strongly encouraged to use pipes (ex: pipe(2) or pipe2(2)) for
+ *            logging fds due to the atomic nature of writes between fds.
+ *            See also libknet test suite for reference and guidance.
  *
  * default_log_level -
  *            If logfd is specified, it will initialize all subsystems to log
@@ -232,9 +238,9 @@ int knet_handle_enable_sock_notify(knet_handle_t knet_h,
  *            socketpair only if they have been created by knet_handle_add_datafd.
  *
  *            It is possible to pass either sockets or normal fds.
- *            User provided datafd will be marked as non-blocking and close-on-exit.
+ *            User provided datafd will be marked as non-blocking and close-on-exec.
  *
- * *channel - This value has the same effect of VLAN tagging.
+ * *channel - This value is analogous to the tag in VLAN tagging.
  *            A negative value will auto-allocate a channel.
  *            Setting a value between 0 and 31 will try to allocate that
  *            specific channel (unless already in use).
@@ -1909,16 +1915,19 @@ const char *knet_log_get_loglevel_name(uint8_t level);
 uint8_t knet_log_get_loglevel_id(const char *name);
 
 /*
- * every log message is composed by a text message (including a trailing \n)
+ * every log message is composed by a text message
  * and message level/subsystem IDs.
  * In order to make debugging easier it is possible to send those packets
  * straight to stdout/stderr (see knet_bench.c stdout option).
  */
 
-#define KNET_MAX_LOG_MSG_SIZE    256
+#define KNET_MAX_LOG_MSG_SIZE    254
+#if KNET_MAX_LOG_MSG_SIZE > PIPE_BUF
+#error KNET_MAX_LOG_MSG_SIZE cannot be bigger than PIPE_BUF for guaranteed system atomic writes
+#endif
 
 struct knet_log_msg {
-	char	msg[KNET_MAX_LOG_MSG_SIZE - (sizeof(uint8_t)*2)];
+	char	msg[KNET_MAX_LOG_MSG_SIZE];
 	uint8_t	subsystem;	/* KNET_SUB_* */
 	uint8_t msglevel;	/* KNET_LOG_* */
 };
