@@ -375,7 +375,7 @@ static int _parse_recv_from_sock(knet_handle_t knet_h, size_t inlen, int8_t chan
 			}
 		}
 	}
-	if ((knet_h->compress_model > 0) && (inlen <= knet_h->compress_threshold)) {
+	if (knet_h->compress_model > 0 && !data_compressed) {
 		knet_h->stats.tx_uncompressed_packets++;
 	}
 
@@ -619,7 +619,7 @@ int knet_send_sync(knet_handle_t knet_h, const char *buff, const size_t buff_len
 out:
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
 
-	errno = savederrno;
+	errno = err ? savederrno : 0;
 	return err;
 }
 
@@ -639,21 +639,11 @@ static void _handle_send_to_links(knet_handle_t knet_h, struct msghdr *msg, int
 	if (inlen == 0) {
 		savederrno = 0;
 		docallback = 1;
-		goto out;
-	}
-	if (inlen < 0) {
-		savederrno = errno;
-		docallback = 1;
-		goto out;
-	}
-
-	knet_h->recv_from_sock_buf->kh_type = type;
-	_parse_recv_from_sock(knet_h, inlen, channel, 0);
-
-out:
-	if (inlen < 0) {
+	} else if (inlen < 0) {
 		struct epoll_event ev;
 
+		savederrno = errno;
+		docallback = 1;
 		memset(&ev, 0, sizeof(struct epoll_event));
 
 		if (epoll_ctl(knet_h->send_to_links_epollfd,
@@ -663,7 +653,9 @@ out:
 		} else {
 			knet_h->sockfd[channel].has_error = 1;
 		}
-
+	} else {
+		knet_h->recv_from_sock_buf->kh_type = type;
+		_parse_recv_from_sock(knet_h, inlen, channel, 0);
 	}
 
 	if (docallback) {
@@ -686,6 +678,8 @@ void *_handle_send_to_links_thread(void *data)
 	struct msghdr msg;
 	struct sockaddr_storage address;
 
+	set_thread_status(knet_h, KNET_THREAD_TX, KNET_THREAD_STARTED);
+
 	memset(&iov_in, 0, sizeof(iov_in));
 	iov_in.iov_base = (void *)knet_h->recv_from_sock_buf->khp_data_userdata;
 	iov_in.iov_len = KNET_MAX_PACKET_SIZE;
@@ -707,7 +701,14 @@ void *_handle_send_to_links_thread(void *data)
 	}
 
 	while (!shutdown_in_progress(knet_h)) {
-		nev = epoll_wait(knet_h->send_to_links_epollfd, events, KNET_EPOLL_MAX_EVENTS + 1, -1);
+		nev = epoll_wait(knet_h->send_to_links_epollfd, events, KNET_EPOLL_MAX_EVENTS + 1, KNET_THREADS_TIMERES / 1000);
+
+		/*
+		 * we use timeout to detect if thread is shutting down
+		 */
+		if (nev == 0) {
+			continue;
+		}
 
 		if (pthread_rwlock_rdlock(&knet_h->global_rwlock) != 0) {
 			log_debug(knet_h, KNET_SUB_TX, "Unable to get read lock");
@@ -741,5 +742,7 @@ void *_handle_send_to_links_thread(void *data)
 		pthread_rwlock_unlock(&knet_h->global_rwlock);
 	}
 
+	set_thread_status(knet_h, KNET_THREAD_TX, KNET_THREAD_STOPPED);
+
 	return NULL;
 }
